;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xdbdb3dd917d853740dffd145562da46a . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x15eb9d1ad61537a7667f1f0cad1dcda4 . "./code.cg2js/cg-printing.fasl")
(#xa85bcfc77965dded78b8a6be8449b1b1 . "./code.cg2js/cg-caret.fasl")
(#x87d62b59e3536eb3e1452ce53808e8f0 . "./code.cg2js/cg-item-list.fasl")
(#xa844b3c5dab0d7c82fb28cba8a73a5e7 . "./code.cg2js/cg-progress-indicator.fasl")
(#xdebc54753199f579286be41a3ba990a8 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x57d87df904a5f02add5372eee692d377 . "./code.cg2js/cg-outline.fasl")
(#x669a0dbd55f05eadc68c7fcce3892ae0 . "./code.cg2js/cg-palette.fasl")
(#x9713e1a6549b4abec171c74a463947b2 . "./code.cg2js/cg-selected-object.fasl")
(#xbeebec9d71c60f55dec1dee0fd7bc14c . "./code.cg2js/cg-status-bar.fasl")
(#x0dabd0f64e6bdcc6db3246e460921a61 . "./code.cg2js/cg-wrap-string.fasl")
(#x71b8bf24e3519c4ff1042e28f52d7705 . "./code.cg2js/cg-fill-texture.fasl")
(#x8c43760d149c4b941c8f7d4b8dc39bf4 . "./code.cg2js/cg-layout.fasl")
(#xc1eddb890f0ae70cb6858a3ef2f5d76a . "./code.cg2js/cg-timer.fasl")
(#x189039aec387dfabd715719fce033da0 . "./code.cg2js/cg-dialog-item.fasl")
(#x7b1198cbf5841b238dc2636f2e5e0a41 . "./code.cg2js/cg-button.fasl")
(#x9e882f905c7d5d5bbbc2877494d15b52 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xffef3278bc6d080d83853baf28fc57d1 . "./code.cg2js/cg-tray.fasl")
(#x15147b0759513814c76501a93d7e705a . "./code.cg2js/cg-plot-widget.fasl")
(#x4c32d5ba2fa016b9023240eec019f9ac . "./code.cg2js/cg-pixmap.fasl")
(#x869f2d40d7d74799045c7201140ba562 . "./code.cg2js/cg-font-dialog.fasl")
(#xc1fdc8b9c84691a08756b3136392a7ad . "./code.cg2js/cg-os-widget.fasl")
(#xc03a564099e59bafdb0ce128827c822e . "./code.cg2js/cg-text-or-combo.fasl")
(#x0b1841c13af4cbd89bc85159af54a195 . "./code.cg2js/cg-trackbar.fasl")
(#xf541ea5b68da795b7a7ab00c7e8daf9a . "./code.cg2js/cg-up-down-control.fasl")
(#x90a92adb16ad13bea02df638f95f7a78 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xd7a55626517c09034d6da2256d9b296a . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x709fb7de6a7ad5394a26d8211e904c57 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xa83fe54c28403f75a537e861589e463e . "./code.cg2js/cg-bitmap-stream.fasl")
(#xf33c20d01b9fb8904f78be5a002a76cd . "./code.cg2js/cg-chart-widget.fasl")
(#xe449d84dbda96924b9bebd7a488b1c47 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xdf048d32edf993806b97e753292bdeaa . "./code.cg2js/cg-group-box.fasl")
(#x53c87925572cec9cc2f684fb26ffcd9e . "./code.cg2js/cg-static-text.fasl")
(#xd492330fdf75adc81c7e27bacfdcdf47 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x9638b092454e368eece2678dcac1ea0d . "./code.cg2js/cg-color-dialog.fasl")
(#x65f216b731bfdf45ce63082914010607 . "./code.cg2js/cg-intersect.fasl")
(#x3240dcefa0bfe49e68d537fa2fdfe945 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xb4cc3d55c4603e38a127d2e479c12faa . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xf3735e1e523f94e00b2ec00f43edd7a6 . "./code.cg2js/cg-html-dialog.fasl")
(#xf0d3110cf435e3dcc7197023fe78bd22 . "./code.cg2js/cg-lettered-menu.fasl")
(#xa70a4a33ba699fe42cdc9993e76456c2 . "./code.cg2js/cg-class-grid.fasl")
(#xbfbbcd500f4bc5c925908640a4ce30fd . "./code.cg2js/cg-pixmap-printing.fasl")
(#x3cd2a97dad7d6ff87608e4ffe2b5a5f0 . "./code.cg2js/cg-calendar.fasl")
(#xee0afb53b6419b50a8b9f2a25029770f . "./code.cg2js/cg-editable-text.fasl")
(#xf1a31d4f83a3e789c9a4aa4a8239967e . "./code.cg2js/cg-property.fasl")
(#x0df2e06623d52b295218db57daad2dc6 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x329396e96ded08de6929aca2b51f8a03 . "./code.cg2js/cg-grid-widget.fasl")
(#xad930e19d52864fe926a25bd997231fe . "./code.cg2js/cg-paren-matching.fasl")
(#xad1c1134a42b7f00fde3c5a2aa870811 . "./code.cg2js/cg-dev.fasl")
(#x56441fa4bb74bef3d37a1bd866071581 . "./code.cg2js/cg-drawable.fasl")
(#xba34edad154c65af04e45e8c13f8b219 . "./code.cg2js/cg-curve.fasl")
(#xbcff5f842226baeec2313e96dab6aa40 . "./code.cg2js/cg-lisp-widget.fasl")
(#xe8b772740e634fc8822f0746853164b8 . "./code.cg2js/cg-dde-utils.fasl")
(#x43b27061cd70ce8353483bfa904e9a5a . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x5aa09974bcd2331cfc16a719ed806b30 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x42751aa69541a5b757c879882fb04e53 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xebd137abb006d21a48d3f251066345d3 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x0c1f17d17b20cb504358a7ee37ebcc8d . "./code.cg2js/cg-lamp.fasl")
(#x30d1beb80097b9f2963ec329a290cd85 . "./code.cg2js/cg-common-control.fasl")
(#x6aba2470dd5b308d6ee3f0b54f24aa80 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xb51379d109d4fc63f979c5b01c1eefab . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x20b371ba3ce068d22970463674f14542 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x096feb50a10c7f992d938f58a0c2213e . "./code.cg2js/cg-project-window.fasl")
(#x2e21ca2d0411851b7d3bdc43a8e9099c . "./code.cg2js/cg-sample-file-menu.fasl")
(#x659a69b396f2ba0f84e069ccaf701207 . "./code.cg2js/cg-class-support.fasl")
(#x89db44bd89df60423768715829b96843 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x0781b3d95546e807faddc6cc48a72d8e . "./code.cg2js/cg-yes-no-list.fasl")
(#x8cdbda834cf58938bc26928737998780 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xef47bd6717422ae27e06cad66f478ee4 . "./code.cg2js/cg-string-dialog.fasl")
(#xa0c586c6a7322d6f13f5d125dd15de94 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xb1a766b6cfa1de86d8d2470faef935e0 . "./code.cg2js/cg-dropping-outline.fasl")
(#xf76f332a73be3b30fc73d10279baf609 . "./code.cg2js/cg.fasl")
(#x240d01825b59415a72635325f77a5ad8 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x5b0f4d1156a30a851a84b2692587fc7c . "./code.cg2js/cg-scaling-stream.fasl")
(#x72a2bf16a7c0ef71c33f3894f9632d71 . "./code.cg2js/cg-yes-no-string.fasl")
(#xc9b3f3c28d80d80c7a41a2a8ae39dfac . "./code.cg2js/cg-icon-pixmap.fasl")
(#xd72315b2291f8b7d5786d946baff3e07 . "./code.cg2js/cg-web-browser.fasl")
(#x729362fa133c2c14f39bbf4ae4fda479 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x20c67244ee9e05efe1d89b4aae1faa67 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x6902834acb4946f6ea61eb7a599d40e0 . "./code.cg2js/cg-message-dialog.fasl")
(#xaa3702444480d0948c641f52e5e356c7 . "./code.cg2js/cg-comtab.fasl")
(#x90d21d63b080a9e986067c50973e88de . "./code.cg2js/cg-base.fasl")
(#xc7b7817030a0b6ddbfd51e4df4b899dc . "./code.cg2js/cg-pixmap-widget.fasl")
(#xbedc11c385834b3da394613389094217 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xcdcbd6f789db4710e83ceaa1de0be4b7 . "./code.cg2js/cg-utility-dialog.fasl")
(#x3b94011beb18bb262cf7f032a0459453 . "./code.cg2js/cg-object-editor.fasl")
(#x6f2d5bee8138e4be03418112bbd244d5 . "./code.cg2js/cg-edit-in-place.fasl")
(#xfd26508b9acb7484263027d94b65af14 . "./code.cg2js/cg-radio-button.fasl")
(#xff9dc97d850679ed37939de38ebf9913 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x8ec908877bd071f46065276c9a6352a5 . "./code.cg2js/cg-nodes.fasl")
(#x619c5e67c35a9a185b0de3189a0180fb . "./code.cg2js/cg-get-position.fasl")
(#xefcdeb06b051e439d1f01a93c7ec7849 . "./code.cg2js/cg-directory-dialog.fasl")
(#xff5944be41fe478550848d3d2765885f . "./code.cg2js/cg-lisp-text.fasl")
(#x4d1dc0a31a39ee7287fde2cbaec3c2d5 . "./code.cg2js/cg-combo-box.fasl")
(#x8fe4ac41f4c2592716274778fe56e02a . "./code.cg2js/cg-text-widget.fasl")
(#x96f5dcf853c1921e0562dc30187c1a5f . "./code.cg2js/cg-list-view.fasl")
(#xf7b4930c7f9eb1efc2ecff560205cfbb . "./code.cg2js/cg-graphics-context.fasl")
(#x19fdc1594cb1c8dc6639c128dfa233f9 . "./code.cg2js/cg-get-pixmap.fasl")
(#x53a78ebe02da16e5fc378d9ad0bfdb59 . "./code.cg2js/cg-menu.fasl")
(#x40679dc3de833cbe36d21d17400bf923 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xac68531156f065c0199e4399d2999e50 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xf36938fafa2aa7d3710659c66b426e61 . "./code.cg2js/cg-tooltip.fasl")
(#xb2356c92056f2f1d1a3dc583264e4cd4 . "./code.cg2js/cg-html-widget.fasl")
(#x9444ac0633f0aea4a28b6f9427983d5a . "./code.cg2js/cg-choice-list.fasl")
(#x2bf286ccfeb6f4bb872a7ff83582a67a . "./code.cg2js/cg-icon.fasl")
(#x5f65396fda3be659ca775a3d186ce56e . "./code.cg2js/cg-acache.fasl")
(#xc5a50afc9364c1c58534773c1e49c964 . "./code.cg2js/cg-template-string.fasl")
(#x4bed0a213c8149755d3ee5d5b75dc858 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x544dc0b5f8c5215f7d8a9e8d9a577325 . "./code.cg2js/cg-toggling-widget.fasl")
(#x23657d06b926b2fa4f048fc5083fff67 . "./code.cg2js/cg-tab-control.fasl")
(#xf6ed8f3f2a9889e22d12fd0c4c8c713c . "./code.cg2js/cg-split-bar.fasl")
(#x745e1c42f50a87a504c734f02f03c753 . "./code.cg2js/cg-picture-widget.fasl")
(#x9909c5026d3c3f0c2e200c876f2e4700 . "./code.cg2js/cg-player.fasl")
(#xc17f555279c0161067ba87356c258414 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x2c7695a6c2fd4fe4b3dae039c079802f . "./code.cg2js/cg-choose-printer.fasl")
(#x93e520eaa62abe1e8d551a27e64b241a . "./code.cg2js/cg-os-window.fasl")
(#xab52080393225189eed7e4bc0113430c . "./code.cg2js/cg-scroll-bar.fasl")
(#x6853aaaef79d91ee41e9a9ee58a709f7 . "./code.cg2js/cg-clipboard.fasl")
(#x834740b1f7f204c0f701283363338b97 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xfd9ce90937885e38b0873dfd9c863afe . "./code.cg2js/cg-hotspot.fasl")
(#x0b592642494b8dde609eaa362968e721 . "./code.cg2js/cg-check-box.fasl")
(#x74b71f4f83e8d783da43bbd8afdc9794 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xf6e7a8d042cfbbc0c64882f7eeed05ee . "./code.cg2js/cg-file-dialog.fasl")
(#x0657937e05aadb7f49a21850a238e891 . "./code.cg2js/cg-toolbar.fasl")
(#x461d527f706922afc7c33a763e1e0a18 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xa40981db96a502cef0fc77129e17be44 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x791d8ae26cb9edf95c81ca39384ee596 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x7190684e238d53a4326062e8afd79a1c . "./code.idejs/ide-project.fasl")
(#x0be991921c7b9168d0725cfd4bb176ce . "./code.idejs/ide-intro.fasl")
(#x8428e2a00dcea233d310e3b474b7b1ae . "./code.idejs/ide-find-in-files.fasl")
(#xee317ba61e277118773d95d08271d47d . "./code.idejs/ide-grapher.fasl")
(#xe51a55a6c3e58eb744789fd3a5e9bddd . "./code.idejs/ide-inspector.fasl")
(#x8110cbaa400b718d27c947f5fab4bab7 . "./code.idejs/ide-editor.fasl")
(#x496aa6b98a18268746a37856ab641af0 . "./code.idejs/ide-window-list.fasl")
(#x5b31932205eed748dd137f19b474f455 . "./code.idejs/ide-process-browser.fasl")
(#x85f8ff0d24ee03d5f49c886820cff9f6 . "./code.idejs/ide-patch-dialog.fasl")
(#x31f31f248c097eb12a8679c8afb1b257 . "./code.idejs/ide-class-grapher.fasl")
(#x652691e273becdb1ff647902e3ec0db5 . "./code.idejs/ide-base.fasl")
(#x6f459f021989f61bfb39dbdb24c86901 . "./code.idejs/ide.fasl")
(#x995d21269d068794339e38dfd1395854 . "./code.idejs/ide-trace.fasl")
(#x6b48d144d87c516b5c2e499b955a59eb . "./code.idejs/ide-form.fasl")
(#x3e0e03199ed2d6c06ac991df4fb8c1ea . "./code.idejs/ide-find-definitions.fasl")
(#x565b4cf44203549d524cc16c0c4f9134 . "./code.idejs/ide-apropos.fasl")
(#x4b9968e20253ebf739aa9ef3b59ef223 . "./code.idejs/ide-class-browser.fasl")
(#xd8c4ac22aa2d470bdae7035b211a2968 . "./code.idejs/ide-navigator.fasl")
(#x720620a8dee93dc8bb3a660ae2c7abfd . "./code.idejs/ide-clipboard.fasl")
(#x0a4be024e99c77f3776d79154901299b . "./code.idejs/ide-acache.fasl")
(#xb574c031cb2705fddeaea1674363ed72 . "./code.idejs/ide-keyshort.fasl")
(#x379c66fd64c2435452efe3aaa0840541 . "./code.idejs/ide-package-list.fasl")
(#x9d954840a08a57cc6c7947064ab9fc34 . "./code.idejs/ide-dev.fasl")
(#x4a9bad5a7becdca3247e09ac504560a1 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
